#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.eco - Exercice API SNCF
# 
# Manipulation d'une [API REST](https://fr.wikipedia.org/wiki/Representational_state_transfer), celle de la SNCF est prise comme exemple. Exercices.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# API signifie Application Programming Interface. Le mot le plus important est “interface”, et c’est le mot le plus simple, car nous utilisons tous des interfaces au quotidien. Par exemple une télécommande de la TV. Vous avez un groupe de boutons et interrupteurs qui vous permettent de faire différentes opérations. Vous savez quel bouton correspond à quelle action. Par exemple, vous ne pouvez pas allumer votre TV avec le bouton de volume. Pour que cela marche, vous devez respecter l'interface et interagir avec elle de la façon qui a été prévue lors de sa conception.
# 
# Une API est une interface pour les applications. Une API vient presque toujours avec une documentation. La documentation du code est un texte écrit par les développeurs qui rend plus facile l’utilisation du code de cette API. Elle explique comment le code fonctionne, pourquoi il a été écrit d’une certaine façon et pas d’une autre, comment contribuer au projet, et bien plus encore. Lire la documentation est essentiel pour bien intégrer l’API d’une autre plateforme. Et chaque API a sa propre logique de fonctionnement. 
# 
# Ici, vous allez découvrir l'API de la SNCF qui donne accès à de nombreuses informations liées à la mobilité.

# Ici, comme souvent, il s'agit d'une API [REST](Comment fonctionne une API sans Python ? Ici, on a une api  classique). REST signifie “Representational State Transfer”. Les API REST sont basées sur HTTP, qui signifie Hypertext Transfer Protocol. C’est le protocole au coeur du web. Il définit la communication entre les différentes parties du web. L’échange est basé sur des requêtes client et serveur. Le principe du client-serveur définit les deux entités qui interagissent dans une API REST : un client et un serveur, les mêmes entités qui communiquent sur le web. Un client envoie une requête, et le serveur renvoie une réponse. 
# 
# Les réponses du serveur pour les API REST peuvent être délivrées dans de multiples formats. JSON (JavaScript Object Notation) est souvent utilisé, mais XML, CSV, ou même RSS sont aussi valables. Ici (comme souvent), il s'agit d'un format JSON.

# ## Partie 0 - modules recommandés et connexion à l'API 
# 
# Il vous faudra sûrement les modules suivant : 
# 
# - [requests](http://docs.python-requests.org/en/master/)
# - [datetime](https://docs.python.org/3/library/datetime.html)
# - [pandas](http://pandas.pydata.org/)
# - [matplotlib](http://matplotlib.org/)
# 
# Créer un login pour vous connecter à l'API de la SNCF https://data.sncf.com/api.
# 
# Il est possible de stocker une clé de façon cachée sur votre ordinateur avec le module [keyring](http://pythonhosted.org/keyring/).

# In[2]:


import os
from pyquickhelper.loghelper import get_password
key = get_password("sncf", "ensae_teaching_cs,key")


# ### Requêter une API sans Python 

# Pour faire une requếté via l'API SNCF, il "suffit" de passer les bons paramètres dans l'url, au format indiqué par la documentation. Comme souvent, c'est un système clé/valeur : url?key1=value1&key2=value2. 
# 
# Pour savoir quelles clés et quelles valeurs indiquer : il faut lire la [Documentation](http://doc.navitia.io/) (la SNCF n'a pas fait sa propre documentation, mais il suffit de remplacer navitia.io par sncf.com dans les exemples donnés).
# 
# En réponse à la requête, le serveur renvoie un document JSON (JavaScript Object Notation). Cela ressemble à un dictionnaire Python.
# 
# Exemple, aller à : https://api.sncf.com/v1/coverage/sncf/stop_areas (si cela ne fonctionne pas, indiquez votre token de la manière suivante :  https://{token}@api.sncf.com/v1/coverage/sncf/stop_areas. Exemple : [stop_areas.json](https://raw.githubusercontent.com/sdpython/ensae_teaching_cs/master/_doc/notebooks/td2a_eco/stop_areas.json).
# 
# Enregistrez le résultat dans un fichier texte (au même endroit où vous avez ouvert votre notebook tant qu'à faire), et ajoutez l'extension .json

# ### Manipuler un fichier json en Python 

# Nous exliquons ci-dessous comment importer et exporter un fichier json en Python.

# In[3]:


import json

with open('stop_areas.json','r') as f:
    data=json.load(f)
data


# Ce n'est pas très lisible. Utilisez le module pretty print.

# In[4]:


import pprint

pprint.pprint(data)


# Avec du javascript.

# In[5]:


from jyquickhelper import JSONJS
JSONJS(data)


# Pour exporter des données en json

# In[6]:


with open('./data_export.json','w') as f:
    json.dump(data,f)


# Il est aussi possible de passer par une chaine de caractères. Exemple :

# In[7]:


import json

data = {
    'name' : 'Mewtwo',
    'pokedex_id' : 150,
    'type' : 'psychic',
    'location':None,
    'best':True
}

#L'encoding JSON gère les types None, bool, int, float, str, lists, tuples et dictionaires.
#L'encoding presque identique à celui de Python, à quelques différences près : 
#None <-> null
#True <-> true
#False <-> false

json_str=json.dumps(data)
print("JSON :", json_str)

data= json.loads(json_str)
print("DATA :", data)


# Il est possible de trier les clés d'un JSON.

# In[8]:


print(json.dumps(data,sort_keys=True))


# ### Requêter une API avec Python 

# In[9]:


import requests
import pprint

#token_auth='votre token ici, recu dans votre boite mail'
token_auth=key

url_avec_params='https://api.sncf.com/v1/coverage/sncf/stop_areas'

r=requests.get(url_avec_params,auth=(token_auth,''))

pprint.pprint(r.json())


# In[10]:


JSONJS(r.json())


# A vous de jouer.

# ## Partie 1 - Trouver les gares accessibles _via_ la SNCF
# 
# - Trouver l'ensemble des gares disponibles sur l'API et créer un fichier csv avec les codes de la gare, son nom et ses coordonnées latitude et longitude, ainsi que les informations administratives de la région quand elles sont disponibles
# 
# - Représentez les sur un graphique

# ## Les trajets depuis la Gare de Lyon

# ### Partons à Lyon : le 17 novembre 2016 à 19h57
# 
# Imaginez que vous vouliez un peu voyager hors de Paris, et il se trouve que justement on vous propose de passer quelques jours à Lyon. Vous partez le 17 novembre vers 19h50 pour ne pas trop écourter votre journée de travail. 

# #### Question 1
# 
# - Commencez par récupérer les informations sur le trajet entre Paris Gare de Lyon et Lyon Perrache le 17 novembre à 19h57
# 
#     - Paris - Gare de Lyon (code de la gare : __stop\_area:OCE:SA:87686006__)
# 
#     - Lyon - Gare Lyon Perrache (code de la gare : __stop\_area:OCE:SA:87722025__)
#     
#     - Indice : utiliser la requête "journeys"
#     
#     - Autre indice : le format de la date est AAAAMMJJTHHMMSS (Année, mois, jour, heure, minutes, secondes)
#     
# - Répondez aux questions suivantes 
#     - combien y a-t-il d'arrêts entre ces deux gares ? (utilisez la clé 'journeys')
#     - combien de temps d'arrêt à chacune d'elles ?

# #### Question 2
# Vous êtes un peu pressé et vous avez peur de vous tromper en arrivant à la gare car d'autres TGV partent à peu près en même temps (à partir de 19h00) de la gare de Lyon. 
# 
# - Si vous demandez à l'API, combien de résultats vous donne-t-elle ?

# - Quels sont les horaires de départ de ces trains ? 

# - Parmi ces trains, combien de trains ont pour destination finale Lyon et qui partent le 17 novembre ?

# ### C'est quand qu'on va où ?
# 
# - En fait, vous n'êtes plus très sûr de vouloir aller à Lyon. Mais bon maintenant vous êtes Gare de Lyon et il est 18h00. 

# #### Question 3
# 
# - Combien de tgv partent entre 18h00 et 20h00 ?
# - Lequel arrive le plus tôt à sa destination finale ?

# ### Et les correspondances ? 

# #### Question 4
# 
# - On va essayer de voir jusqu'où on peut aller, en prenant des trains au départ de la Gare de Lyon : 
#     - Quelles sont toutes les gares atteignables en partant le 17 novembre, sans faire de changement et sans partir après minuit ?
#     - Si on prend un de ces trains, jusqu'où peut-on aller, avec une correspondance, sans partir après 8h le lendemain matin ?

# ##### Etape 1 : trouver toutes les correspondances possibles depuis le trajet entre les gares de Paris et de Perpignan
# 
# Essayer de trouver toutes les correspondances possibles depuis un trajet entre Paris et Perpignan

# ##### Etape 2 : Généraliser et trouver toutes les correspondances possibles depuis les trains qu'on prend de la Gare de Lyon, le 17 novembre au soir
# 
# Maintenant qu'on a fait un exemple, on le fait pour tous les trajets qui partent de la Gare de Lyon entre 18h et 20h de la Gare de Lyon
# 
# Les correspondances elles devront partir avant 8h00 du matin le lendemain

# #### Question 5
# - Représenter toutes les gares atteignables avec un graphique type scatter. Distinguer les gares atteintes en un seul trajet et celles atteintes avec une correspondance.

# In[11]: